SystemOrganization addCategory: #'Polymorph-Tools-Diff'!

!MCMergeResolutionRequest methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 2/7/2009 11:26'!
defaultAction
	"Modally open a merge tool."
	
	^self viewMerger! !

!MCMergeResolutionRequest methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 2/9/2010 14:06'!
viewMerger
	"Open a model browser to perform the merge and answer wheter merged."

	^PSMCPatchMorph usedByDefault
		ifTrue: [self viewPatchMerger]
		ifFalse: [(MCMergeBrowser new
					merger: merger;
					label: messageText) showModally]! !

!MCMergeResolutionRequest methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 2/4/2010 16:48'!
viewPatchMerger
	"Open a modal diff tools browser to perform the merge."

	|m modalMorph|
	m := (PSMCMergeMorph forMerger: self merger)
		fromDescription: 'Working copy' translated;
		toDescription: messageText.
	modalMorph := (UIManager default respondsTo: #modalMorph)
		ifTrue: [UIManager default modalMorph]
		ifFalse: [World].
	modalMorph openModal: (
		m newWindow
			title: messageText).
	^m merged! !

!MCMethodDefinition methodsFor: '*Polymorph-Tools-Diff' stamp: 'BenjaminVanRyseghem 9/28/2011 15:24'!
shortSummaryPrefix

	^ self definition selector asString! !

ComposableMorph subclass: #DiffChangeMorph
	instanceVariableNames: 'diffMorph descriptionMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!DiffChangeMorph class methodsFor: 'as yet unclassified' stamp: 'gvc 2/9/2010 13:10'!
from: old label: oldLabel to: new label: newLabel contextClass: aClass
	"Answer a new instance of the receiver with the given
	old and new text and descriptions."

	^self new
		from: old
		to: new
		contextClass: aClass;
		fromDescription: oldLabel;
		toDescription: newLabel! !

!DiffChangeMorph methodsFor: 'actions' stamp: 'gvc 2/9/2010 13:07'!
adoptPaneColor: paneColor
	"Change the fill styles on the descriptions."
	
	super adoptPaneColor: paneColor.
	paneColor ifNil: [^self].
	self descriptionMorph borderStyle: (self theme listNormalBorderStyleFor: self).
	self updateDescriptionFillStyle: paneColor
	! !

!DiffChangeMorph methodsFor: 'accessing' stamp: 'gvc 2/9/2010 13:12'!
defaultTitle
	"Answer the default title label for the receiver."

	^'Change Diff' translated! !

!DiffChangeMorph methodsFor: 'accessing' stamp: 'gvc 2/9/2010 13:07'!
descriptionMorph
	"Answer the value of descriptionMorph"

	^ descriptionMorph! !

!DiffChangeMorph methodsFor: 'accessing' stamp: 'gvc 2/9/2010 13:07'!
descriptionMorph: anObject
	"Set the value of descriptionMorph"

	descriptionMorph := anObject! !

!DiffChangeMorph methodsFor: 'accessing' stamp: 'gvc 2/9/2010 13:07'!
diffMorph
	"Answer the value of diffMorph"

	^ diffMorph! !

!DiffChangeMorph methodsFor: 'accessing' stamp: 'gvc 2/9/2010 13:07'!
diffMorph: anObject
	"Set the value of diffMorph"

	diffMorph := anObject! !

!DiffChangeMorph methodsFor: 'hooks' stamp: 'gvc 2/9/2010 13:12'!
diffMorphClass
	"Answer a the class to use for a new diff morph."

	^DiffMorph! !

!DiffChangeMorph methodsFor: 'actions' stamp: 'gvc 2/9/2010 13:10'!
from: old to: new contextClass: aClass
	"Set the old (src) and new (dst) text."

	self diffMorph
		from: old
		to: new
		contextClass: aClass! !

!DiffChangeMorph methodsFor: 'actions' stamp: 'gvc 2/9/2010 13:12'!
fromDescription: aString
	"Set the description for the left-hand side of the patch,
	typically 'currently in image'."

	self fromDescriptionMorph contents: aString! !

!DiffChangeMorph methodsFor: 'actions' stamp: 'gvc 2/9/2010 13:12'!
fromDescriptionMorph
	"Answer the morph for the source description."

	^self descriptionMorph firstSubmorph firstSubmorph! !

!DiffChangeMorph methodsFor: 'initialize-release' stamp: 'gvc 2/9/2010 13:12'!
initialColorInSystemWindow: aSystemWindow
	"Answer the colour the receiver should be when added to a SystemWindow."

	^Color transparent! !

!DiffChangeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 3/5/2010 12:43'!
initialExtent
	"Answer the initial extent for the receiver."

	^RealEstateAgent standardWindowExtent! !

!DiffChangeMorph methodsFor: 'initialize-release' stamp: 'gvc 2/9/2010 13:52'!
initialize
	"Initialize the receiver."

	|descriptionHeight|
	super initialize.
	self
		diffMorph: self newDiffMorph;
		descriptionMorph: self newDescriptionMorph.
	descriptionHeight := self descriptionMorph minExtent y.
	self
		changeProportionalLayout;
		addMorph: self descriptionMorph
		fullFrame: (LayoutFrame fractions: (0@0 corner: 1@0) offsets: (0@0 corner: 0@descriptionHeight));
		addMorph: self diffMorph
		fullFrame: (LayoutFrame fractions: (0@0 corner: 1@1)
					offsets: (0 @ descriptionHeight corner: 0@0)).
	self
		extent: self initialExtent;
		updateDescriptionFillStyle: self paneColor! !

!DiffChangeMorph methodsFor: 'actions' stamp: 'gvc 2/9/2010 13:17'!
newDescriptionMorph
	"Answer a new morph for the descriptions of the source and destination."

	^(self newColumn: {
		(self newRow: {self newLabel: 'Original'}) layoutInset: 1.
		(self newRow: {self newLabel: 'Changed'})
			layoutInset: 1;
			listCentering: #bottomRight})
		layoutInset: 0;
		cellInset: 2;
		fillStyle: Color white;
		borderStyle: (self theme listNormalBorderStyleFor: self)
		! !

!DiffChangeMorph methodsFor: 'actions' stamp: 'gvc 2/9/2010 13:12'!
newDiffMorph
	"Answer a new morph for the source difference."

	^self diffMorphClass new
		borderStyle: (BorderStyle inset width: 1);
		font: self theme statusFont;
		addDependent: self;
		yourself! !

!DiffChangeMorph methodsFor: 'actions' stamp: 'gvc 2/9/2010 13:13'!
toDescription: aString
	"Set the description for the right-hand side of the patch,
	typically 'incoming' or 'changed'."

	self toDescriptionMorph contents: aString! !

!DiffChangeMorph methodsFor: 'actions' stamp: 'gvc 2/9/2010 13:13'!
toDescriptionMorph
	"Answer the morph for the destination description."

	^self descriptionMorph lastSubmorph firstSubmorph! !

!DiffChangeMorph methodsFor: 'actions' stamp: 'gvc 2/9/2010 13:13'!
updateDescriptionFillStyle: aColor
	"Change the colours on the descriptions."
	
	self descriptionMorph fillStyle: aColor.
	self fromDescriptionMorph owner color: (aColor alphaMixed: 0.5 with: Color white).
	self toDescriptionMorph owner color: (aColor alphaMixed: 0.3 with: Color white).! !

DiffChangeMorph subclass: #PSMCPatchMorph
	instanceVariableNames: 'changeTree selectedChangeWrapper'
	classVariableNames: 'UsedByDefault'
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!PSMCPatchMorph commentStamp: 'LaurentLaffont 2/23/2011 20:21' prior: 0!
I'm a Monticello tool based on polymorph used to browse diffs.
Activate me with the setting named 'Use Polymorph difference tools'.!

PSMCPatchMorph subclass: #PSMCMergePatchMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 2/25/2010 10:21'!
allConflicts
	"Answer all conflicts in the model."

	^self model select: [:op | op isConflict]! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'GuillermoPolito 5/29/2011 14:51'!
allConflictsMenu
	"Answer the all conflicts menu."

	|menu|
	menu := UIManager default newMenuIn: self for: self.
		menu
		addToggle: 'Keep current version of all' translated
		target: self
		selector: #keepAllCurrentVersion
		getStateSelector: nil
		enablementSelector: #hasAnyNonKeptConflicts.
	menu lastItem
	 	font: self theme menuFont;
		icon: self theme smallBackIcon;
		keyText: 'Shift+Cmd+c'.	
	menu
		addToggle: 'Use incoming version of all' translated
		target: self
		selector: #useAllIncomingVersion
		getStateSelector: nil
		enablementSelector: #hasAnyNonIncomingConflicts.
	menu lastItem
	 	font: self theme menuFont;
		icon: self theme smallForwardIcon;
		keyText: 'Shift+Cmd+g'.
	menu
		addToggle: 'Mark all as conflict' translated
		target: self
		selector: #markAllAsConflict
		getStateSelector: nil
		enablementSelector: #hasAnyNonConflictConflicts.
	menu lastItem
	 	font: self theme menuFont;
		icon: self theme smallCancelIcon;
		keyText: 'Shift+Cmd+x'.
	^menu! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'StephaneDucasse 6/24/2011 16:08'!
changeTreeKey: aCharacter
	"Handle change tree key strokes."

	super changeTreeKey: aCharacter.
	(aCharacter == $a and: [ self selectionIsNotNil]) ifTrue: [
		^ self toggleApplySelectedChange.
		].
	(aCharacter == $c and: [self selectionIsNotNil]) ifTrue: [
		^self keepCurrentVersion].
	(aCharacter == $g and: [self selectionIsNotNil]) ifTrue: [
		^self useIncomingVersion].
	(aCharacter == $x and: [self selectionIsNotNil]) ifTrue: [
		^self markAsConflict].
	(aCharacter == $f and: [self notAllConflictsResolved]) ifTrue: [
		^self selectNextConflict].
	
	(aCharacter == $C and: [self hasAnyNonKeptConflicts]) ifTrue: [
		^self keepAllCurrentVersion].
	(aCharacter == $G and: [self hasAnyNonIncomingConflicts]) ifTrue: [
		^self useAllIncomingVersion].
	(aCharacter == $X and: [self hasAnyNonConflictConflicts]) ifTrue: [
		^self markAllAsConflict]! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 6/24/2011 17:31'!
changesMenu: m
	"Answer the changes menu."

	|menu|
	menu := super changesMenu: m.
	menu addLine.
	menu addToggle: 'Apply this change' translated
		target: self
		selector: #toggleApplySelectedChange
		getStateSelector: #selectionIsRemoteChosen
		enablementSelector: #selectionIsNotNil.
	menu
		addToggle: 'Keep current version' translated
		target: self
		selector: #keepCurrentVersion
		getStateSelector: nil
		enablementSelector: #selectionIsNotNil.
	menu lastItem
	 	font: self theme menuFont;
		icon: self theme smallBackIcon;
		keyText: 'Cmd+c'.	
	menu
		addToggle: 'Use incoming version' translated
		target: self
		selector: #useIncomingVersion
		getStateSelector: nil
		enablementSelector: #selectionIsNotNil.
	menu lastItem
	 	font: self theme menuFont;
		icon: self theme smallForwardIcon;
		keyText: 'Cmd+g'.
	menu
		addToggle: 'Mark as conflict' translated
		target: self
		selector: #markAsConflict
		getStateSelector: nil
		enablementSelector: #selectionIsNotNil.
	menu lastItem
	 	font: self theme menuFont;
		icon: self theme smallCancelIcon;
		keyText: 'Cmd+x'.
	menu
		addToggle: 'All conflicts' translated
		target: self
		selector: nil
		getStateSelector: nil
		enablementSelector: #hasConflicts.
	menu lastItem
	 	font: self theme menuFont;
		subMenu: self allConflictsMenu.
	menu addLine.
	menu
		addToggle: 'Select next conflict' translated
		target: self
		selector: #selectNextConflict
		getStateSelector: nil
		enablementSelector: #notAllConflictsResolved.
	menu lastItem
	 	font: self theme menuFont;
		icon: self theme smallRightFlushIcon;
		keyText: 'Cmd+f'.	
	^menu! !

!PSMCMergePatchMorph methodsFor: 'hooks' stamp: 'gvc 11/1/2006 14:25'!
diffMorphClass
	"Answer a the class to use for a new diff morph."

	^MergeDiffMorph! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 2/25/2010 13:51'!
hasAnyNonConflictConflicts
	"Answer whether there are any conflicts not marked as a conflict."

	^self allConflicts anySatisfy: [:conflict |
		conflict isResolved]! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 2/25/2010 13:50'!
hasAnyNonIncomingConflicts
	"Answer whether there are any conflicts not marked as incoming."

	^self allConflicts anySatisfy: [:conflict |
		conflict isResolved not or: [conflict localChosen]]! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 2/25/2010 13:51'!
hasAnyNonKeptConflicts
	"Answer whether there are any conflicts not marked as keep."

	^self allConflicts anySatisfy: [:conflict |
		conflict isResolved not or: [conflict remoteChosen]]! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 2/25/2010 13:38'!
hasConflicts
	"Answer whether there are any conflicts."

	^self allConflicts notEmpty! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 2/25/2010 10:18'!
keepAllCurrentVersion
	"Mark all conflicts as local."
	
	self allConflicts do: [:c | c chooseLocal].
	self changed: #changes.
	self updateSource! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 7/6/2007 15:46'!
keepCurrentVersion
	"Mark the conflict as local."
	
	self selectedChangeWrapper chooseLocal.
	self changed: #changes.
	self updateSource! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 2/25/2010 10:17'!
markAllAsConflict
	"Mark all conflicts as unresolved."
	
	self allConflicts do: [:c | c clearChoice].
	self changed: #changes.
	self updateSource! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'StephaneDucasse 6/24/2011 16:07'!
markAsConflict
	"Mark the operation as unresolved conflict."
	
	self selectedChangeWrapper operation beConflict.
	self changed: #changes.
	self updateSource.! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 2/9/2010 14:18'!
nextConflict
	"Answer the next (unresolved) conflict or, nil if none."

	|coll current index|
	current := self selectedChangeWrapper.
	index := current isNil
		ifTrue: [0]
		ifFalse: [self model indexOf: current item].
	coll := (self model
			copyFrom: index + 1
			to: self model size),
		(self model copyFrom: 1 to: index).
	^coll detect: [:item | item isConflict and: [item isResolved not]] ifNone: [nil]! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/29/2009 14:26'!
notAllConflictsResolved
	"Answer whether any conflicts are unresolved."
	
	^self model anySatisfy: [:item | item isConflict and: [item isResolved not]]! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 4/2/2009 13:03'!
selectNextConflict
	"Select the next conflict after the current selection, if any."

	|next op def path|
	next := self nextConflict ifNil: [^self].
	op := next operation.
	def := next remoteDefinition ifNil: [next localDefinition].
	path := {#changes. #openPath}.
	def isMethodDefinition ifTrue: [path := path, {def fullClassName}].
	path := path, {op shortSummary}.
	self changed: path! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 6/22/2011 17:51'!
selectionIsNotNil

	^ self selectedChange notNil! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 6/24/2011 17:30'!
selectionIsRemoteChosen
	self selectedChange ifNil: [ ^ false].
	^ self selectedChange remoteChosen! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 6/24/2011 17:28'!
toggleApplySelectedChange

	self selectedChange remoteChosen 
		ifTrue: [ self selectedChange chooseLocal ]
		ifFalse: [ self selectedChange chooseRemote ].
	self changed: #changes.
	self updateSource! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/12/2009 14:06'!
updateSource
	"Update the source difference morph."

	|sel|
	sel := self selectedChangeWrapper.
	self diffMorph allowJoinClicks: (sel notNil and: [
		sel isConflict and: [sel operation isModification]]).
	super updateSource.
	(sel isNil or: [sel isConflict not]) ifTrue: [^self].
	sel localChosen
		ifTrue: [self diffMorph indicateSrc]
		ifFalse: [self diffMorph indicateDst]! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 2/25/2010 10:18'!
useAllIncomingVersion
	"Mark all conflicts as remote."
	
	self allConflicts do: [:c | c chooseRemote].
	self changed: #changes.
	self updateSource! !

!PSMCMergePatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 7/6/2007 15:46'!
useIncomingVersion
	"Mark the conflict as remote."
	
	self selectedChangeWrapper chooseRemote.
	self changed: #changes.
	self updateSource! !

!PSMCPatchMorph class methodsFor: 'as yet unclassified' stamp: 'gvc 10/30/2006 11:08'!
forPatch: aPatch
	"Answer a new instance of the receiver
	with the given patch as the model."
	
	^ self new model: aPatch operations! !

!PSMCPatchMorph class methodsFor: 'as yet unclassified' stamp: 'gvc 2/9/2010 14:04'!
usedByDefault
	"Answer whether the Polymorph diff tools should be used with Monticello."

	^UsedByDefault ifNil: [UsedByDefault := true]! !

!PSMCPatchMorph class methodsFor: 'as yet unclassified' stamp: 'gvc 2/9/2010 14:03'!
usedByDefault: aBoolean
	"Specify whether the Polymorph diff tools should be used with Monticello."
	
	UsedByDefault := aBoolean! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'BenjaminVanRyseghem 2/8/2012 17:09'!
browseClass
	"Browse the class of the selected item."

	^ Smalltalk tools browser
		fullOnClass: self selectedChangeWrapper actualClass
		selector: self selectedMessageName! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/14/2009 12:47'!
browseImplementors
	"Browse the method implementors."

	self systemNavigation
		browseAllImplementorsOf: (self selectedMessageName ifNil: [^self])! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/14/2009 12:47'!
browseSenders
	"Browse the method senders."

	self systemNavigation
		browseAllCallsOn: (self selectedMessageName ifNil: [^self])! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'StephaneDucasse 1/21/2012 22:56'!
browseVersions
	"Browse the method versions."

	| aClass aSelector |
	aClass := self selectedChangeWrapper actualClass.
	aSelector := self selectedMessageName ifNil: [^self].
	Smalltalk tools versionBrowser 
			browseVersionsForClass: aClass selector: aSelector
			! !

!PSMCPatchMorph methodsFor: 'accessing' stamp: 'gvc 1/29/2009 13:17'!
changeTree
	"Answer the value of changeTree"

	^ changeTree! !

!PSMCPatchMorph methodsFor: 'accessing' stamp: 'gvc 1/29/2009 13:17'!
changeTree: anObject
	"Set the value of changeTree"

	changeTree := anObject! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 2/22/2010 17:57'!
changeTreeKey: aCharacter
	"Handle change tree key strokes."

	(aCharacter == $b and: [self selectionHasAcutalClass]) ifTrue: [
		^self browseClass].
	(aCharacter == $v and: [self selectionIsMethodChange]) ifTrue: [
		^self browseVersions].
	(aCharacter == $m and: [self selectionIsMethodChange]) ifTrue: [
		^self browseImplementors].
	(aCharacter == $n and: [self selectionIsMethodChange]) ifTrue: [
		^self browseSenders]! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'nice 3/28/2011 23:21'!
changes
	"Answer the changes tree roots."

	|changes classes|
	self model ifNil: [^#()].
	changes := OrderedCollection new.
	classes := Set new.
	self model do: [:o |
		o definition isOrganizationDefinition
			ifTrue: [changes add: (o patchWrapper model: self model)]
			ifFalse: [
				o targetClassName notNil ifTrue:[
				
				(classes includes: o targetClassName)
					ifFalse: [classes add: o targetClassName.
							changes add: (PSMCClassChangeWrapper with: o targetClassName model: self model)]]]].
	^(changes reject: [:e | e isKindOf: PSMCClassChangeWrapper]) , ((changes select: [:e | e isKindOf: PSMCClassChangeWrapper]) sorted: [:a :b | a item <= b item])! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 4/26/2012 10:30'!
changesMenu: m
	"Answer the changes menu."

	|menu|
	menu := self newMenu
		addTitle: 'Changes'
		icon: self theme smallCopyIcon.		
	menu
		addToggle: 'Browse class...' translated
		target: self
		selector: #browseClass
		getStateSelector: nil
		enablementSelector: #selectionHasAcutalClass.
	menu lastItem
	 	font: self theme menuFont;
		icon: Smalltalk tools browser taskbarIcon;
		keyText: 'Cmd+b'.
	menu addLine.
	menu
		addToggle: 'Versions...' translated
		target: self
		selector: #browseVersions
		getStateSelector: nil
		enablementSelector: #selectionIsMethodChange.
	menu lastItem
	 	font: self theme menuFont;
		icon: self theme smallJustifiedIcon;
		keyText: 'Cmd+v'.	
	menu
		addToggle: 'Senders...' translated
		target: self
		selector: #browseSenders
		getStateSelector: nil
		enablementSelector: #selectionIsMethodChange.
	menu lastItem
	 	font: self theme menuFont;
		icon: self theme smallForwardIcon;
		keyText: 'Cmd+n'.	
	menu
		addToggle: 'Implementors...' translated
		target: self
		selector: #browseImplementors
		getStateSelector: nil
		enablementSelector: #selectionIsMethodChange.
	menu lastItem
	 	font: self theme menuFont;
		icon: self theme smallDoItIcon;
		keyText: 'Cmd+m'.
	menu
		addToggle: 'Install incoming version' translated
		target: self
		selector: #loadMethodSelection
		getStateSelector: nil
		enablementSelector: #selectionIsMethodChange.
	menu lastItem
	 	font: self theme menuFont;
		icon: self theme smallUpdateIcon.
	menu
		add: 'Expand All' translated
		target: self model
		selector: #expandAll.
	menu
		add: 'Collapse All' translated
		target: self model
		selector: #collapseAll.
	^menu! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 12:02'!
compositeText
	"Answer the composite text based on the selection state
	of the joins."

	^self diffMorph compositeText! !

!PSMCPatchMorph methodsFor: 'accessing' stamp: 'gvc 10/29/2006 18:18'!
defaultTitle
	"Answer the default title label for the receiver."

	^'Changes' translated! !

!PSMCPatchMorph methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 5/23/2012 12:42'!
initialize
	"Initialize the receiver."

	|descriptionHeight|
	super initialize.
	self
		diffMorph: self newDiffMorph;
		changeTree: self newChangeTreeMorph;
		descriptionMorph: self newDescriptionMorph.
	descriptionHeight := self descriptionMorph minExtent y.
	self
		changeProportionalLayout;
		addMorph: self descriptionMorph
		fullFrame: (LayoutFrame fractions: (0.3@0 corner: 1@0) offsets: (ProportionalSplitterMorph splitterWidth @ 0 corner: 0@descriptionHeight));
		addMorph: self changeTree
		fullFrame: (LayoutFrame fractions: (0@0 corner: 0.3@1) offsets: (0@0 corner: 0@0));
		addMorph: self diffMorph
		fullFrame: (LayoutFrame fractions: (0.3@0 corner: 1@1)
					offsets: (ProportionalSplitterMorph splitterWidth @ descriptionHeight corner: 0@0));
		addPaneSplitters.
	self updateDescriptionFillStyle: self paneColor.
	self splitters first delete.
	self onAnnouncement: MorphOpened do: [self changeTree expandAll]
! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'MarcusDenker 8/15/2010 11:38'!
loadMethodSelection
	"Install the selected change"
	
	self selectedChange ifNil: [ ^self ].
	self selectedChange definition load.! !

!PSMCPatchMorph methodsFor: 'initialization' stamp: 'gvc 10/30/2006 11:08'!
model: aCollection
	"Set the model and update the window."

	super model: aCollection.
	self changed: #changes! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 2/22/2010 17:52'!
newChangeTreeMorph
	"Answer a new morph for the tree of changes."

	^(self
		newTreeFor: self
		list: #changes
		selected: #selectedChangeWrapper
		changeSelected: #selectedChangeWrapper:)
		getMenuSelector: #changesMenu:;
		keystrokeActionSelector: #changeTreeKey:! !

!PSMCPatchMorph methodsFor: 'accessing' stamp: 'gvc 10/26/2006 11:58'!
selectedChange
	"Answer the selected change."

	^(self selectedChangeWrapper ifNil: [^nil]) operation! !

!PSMCPatchMorph methodsFor: 'accessing' stamp: 'gvc 10/26/2006 11:57'!
selectedChangeWrapper
	"Answer the selected change."

	^selectedChangeWrapper! !

!PSMCPatchMorph methodsFor: 'accessing' stamp: 'gvc 11/1/2006 12:18'!
selectedChangeWrapper: aWrapper
	"Set the selected change."

	selectedChangeWrapper := aWrapper.
	self
		changed: #selectedChangeWrapper;
		updateSource;
		changed: #compositeText! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/26/2006 15:52'!
selectedMessageName
	"Answer the method selector or nil if no method change
	is selected.."

	^self selectionIsMethodChange
		ifTrue: [self selectedChange definition selector]
		! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 8/12/2009 13:28'!
selectionHasAcutalClass
	"Answer whether the currently selected change has an actual
	class in the image."

	^self selectedChangeWrapper
		ifNil: [false]
		ifNotNilDo: [:w | w actualClass notNil]! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'GuillermoPolito 1/11/2012 22:53'!
selectionIsMethodChange
	"Answer whether the currently selected change is for a method."

	^self selectedChange notNil and: [ 
		self selectedChange definition isMethodDefinition and: [ 
			self selectedChange definition isLoadable ] ]! !

!PSMCPatchMorph methodsFor: 'updating' stamp: 'gvc 10/30/2006 15:30'!
update: aspect
	"A join has probably changed its selection state."

	super update: aspect.
	aspect == #selectedDifferences
		ifTrue: [self changed: #compositeText]! !

!PSMCPatchMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/14/2009 13:28'!
updateSource
	"Update the source difference morph."

	|sel|
	sel := self selectedChange.
	sel isNil
		ifTrue: [self diffMorph
				from: ''
				to: '']
		ifFalse: [self diffMorph
					from: sel diffFromSource
					to: sel diffToSource
					contextClass: (sel isClassPatch ifTrue: [nil] ifFalse: [sel targetClass])]! !

ComposableMorph subclass: #DiffMorph
	instanceVariableNames: 'srcText dstText prettyPrint contextClass srcMorph dstMorph scrollbarMorph mapMorph joinMorph difference joinMappings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!DiffMorph class methodsFor: 'as yet unclassified' stamp: 'gvc 10/20/2006 11:26'!
from: old to: new
	"Answer a new instance of the receiver with the given
	old and new text."

	^self new
		from: old
		to: new! !

!DiffMorph class methodsFor: 'as yet unclassified' stamp: 'gvc 9/2/2008 15:46'!
from: old to: new contextClass: aClass
	"Answer a new instance of the receiver with the given
	old and new text."

	^self new
		from: old
		to: new
		contextClass: aClass! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/24/2006 12:34'!
additionColor
	"Answer the color used to show additions."

	^Color paleGreen alpha: 0.5! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 10/29/2006 18:23'!
adoptPaneColor: paneColor
	"Change our border color too."
	
	super adoptPaneColor: paneColor.
	paneColor ifNil: [^self].
	self borderStyle baseColor: paneColor! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 10:52'!
applyHighlights
	"Apply the relevant highlights to src and dst."

	self srcMorph highlights: (self joinMappings gather: [:j | j src highlights]).
	self dstMorph highlights: (self joinMappings gather: [:j | j dst highlights])! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/24/2006 11:11'!
applyJoin
	"Apply the join mappings to the join morph."

	self joinMorph mappings: self joinMappings! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/26/2006 14:11'!
applyMap
	"Apply the join mappings to the map morph."

	self mapMorph mappings: self joinMappings! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/20/2006 13:11'!
calculateDifference
	"Calculate the difference of the src and dst."

	self difference: ((TextDiffBuilder
		from: self oldText asString to: self newText asString)
			buildPatchSequence)! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/24/2006 15:56'!
calculateJoinMappings
	"Calculate the join parameters between src and dst
	and store in joinMappings."

	self joinMappings: self calculatedJoinMappings! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/31/2006 11:51'!
calculatedJoinMappings
	"Calculate the join parameters between src and dst
	and answer.
	sl = src line, dl = dst line, j = joins, ds = dst run start, ss = src run start
	de = dst run end, se = dst run end, mds = match dst start, mss = match src start"

	|sl dl j ds ss de se mds mss|
	sl := dl := 0.
	j := OrderedCollection new.
	ds := de:=  ss := se := mss := mds := 0.
	self difference do: [:p |
		p key = #match ifTrue: [
			sl := sl + 1.
			dl := dl + 1.
			mss = 0 ifTrue: [mss := sl. mds := dl].
			(ds > 0 or: [ss > 0]) ifTrue: [
				ss = 0 ifTrue: [ss := sl].
				ds = 0 ifTrue: [ds := dl].
				se = 0 ifTrue: [se := ss - 1].
				de = 0 ifTrue: [de := ds - 1].
				j add: (self newJoinSectionFrom: (ss to: se) to: (ds to: de)).
				ds := de := ss := se := 0]].
		p key = #remove ifTrue: [
			mss > 0 ifTrue: [
				j add: (self newMatchJoinSectionFrom: (mss to: sl) to: (mds to: dl)).
				mss := mds := 0].
			sl := sl + 1.
			ss = 0 ifTrue: [ss := sl].
			se := sl].
		p key = #insert ifTrue: [
			mss > 0 ifTrue: [
				j add: (self newMatchJoinSectionFrom: (mss to: sl) to: (mds to: dl)).
				mss := mds := 0].
			dl := dl + 1.
			ss > 0 ifTrue: [
				se = 0 ifTrue: [se := ss].
				de = 0 ifTrue: [de := ds].
				j add: (self newJoinSectionFrom: (ss to: se) to: (ds to: de)).
				ds := de := ss := se := 0].
			ds = 0 ifTrue: [ds := dl].
			de := dl]].
	sl := sl + 1.
	dl := dl + 1.
	(ds > 0 or: [ss > 0]) ifTrue: [
		ss = 0 ifTrue: [ss := sl ].	
		ds = 0 ifTrue: [ds := dl].
		se = 0 ifTrue: [se := ss - 1].
		de = 0 ifTrue: [de := ds - 1].
		j add: (self newJoinSectionFrom: (ss to: se) to: (ds to: de))].
	mss > 0 ifTrue: [
		j add: (self newMatchJoinSectionFrom: (mss to: sl - 1) to: (mds to: dl - 1))].
	^j! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/23/2006 16:29'!
calibrateScrollbar
	"Set the scrollbar parameters to match the texts."

	|maxY range delta innerH|
	self fullBounds.
	maxY := self srcMorph textExtent y max: self dstMorph textExtent y.
	innerH := self dstMorph innerBounds height.
	delta := self dstMorph textMorph defaultLineHeight.
	range := maxY - innerH max: 0.
	range = 0 ifTrue: [^self scrollbarMorph scrollDelta: 0.02 pageDelta: 0.2; interval: 1.0; setValue: 0.0].
	self scrollbarMorph
		scrollDelta: (delta / range) asFloat 
		pageDelta: ((innerH - delta) / range) asFloat;
		interval: (innerH / maxY) asFloat;
		setValue: ((self srcMorph scroller offset y max: self dstMorph scroller offset y)
					 / range min: 1.0) asFloat! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/26/2006 14:29'!
colorForType: type
	"Anwser the color to use for the given change type."

	^{self matchColor. self additionColor. self removalColor. self modificationColor}
		at: (#(match addition removal modification) indexOf: type)! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 9/2/2008 15:38'!
contextClass
	"Answer the value of contextClass"

	^ contextClass! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 9/2/2008 15:38'!
contextClass: anObject
	"Set the value of contextClass"

	contextClass := anObject! !

!DiffMorph methodsFor: 'initialization' stamp: 'gvc 10/26/2006 14:10'!
defaultColor
	"Answer the default color for the receiver."

	^Color white! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 11:04'!
defaultTitle
	"Answer the default title label for the receiver."

	^'Diff' translated! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 10/20/2006 11:27'!
difference
	"Answer the value of difference"

	^ difference! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 10/20/2006 11:27'!
difference: anObject
	"Set the value of difference"

	difference := anObject! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 10/20/2006 10:46'!
dstMorph
	"Answer the value of dstMorph"

	^ dstMorph! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 10/20/2006 10:46'!
dstMorph: anObject
	"Set the value of dstMorph"

	dstMorph := anObject! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/25/2006 18:22'!
dstScroll: scrollValue
	"Called from dst when scrolled by keyboard etc."

	self scrollbarMorph value: scrollValue.
	self srcMorph vScrollBarValue: scrollValue.
	self updateJoinOffsets! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 9/2/2008 15:21'!
dstText
	"Answer the value of dstText"

	^ dstText! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 9/2/2008 15:21'!
dstText: anObject
	"Set the value of dstText"

	dstText := anObject! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/24/2006 12:35'!
edgeColor
	"Answer the color used to show the border of the changes."

	^Color gray alpha: 0.5! !

!DiffMorph methodsFor: 'geometry' stamp: 'gvc 10/23/2006 16:27'!
extent: newExtent
	"Update the scrollbar."

	super extent: newExtent.
	self calibrateScrollbar! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/31/2006 13:23'!
font: aFont
	"Set the font on the src and dst morphs."

	self srcMorph font: aFont.
	self dstMorph font: aFont! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 9/2/2008 15:24'!
from: old to: new
	"Set the old (src) and new (dst) text."

	self
		srcText: old;
		dstText: new.
	self
		setText;
		calculateDifference;
		calculateJoinMappings;
		calibrateScrollbar;
		applyHighlights;
		applyJoin;
		applyMap! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 9/2/2008 15:45'!
from: old to: new contextClass: aClass
	"Set the old (src) and new (dst) text."

	self
		contextClass: aClass;
		srcText: old;
		dstText: new.
	self
		setText;
		calculateDifference;
		calculateJoinMappings;
		calibrateScrollbar;
		applyHighlights;
		applyJoin;
		applyMap! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/23/2006 16:12'!
hideOrShowScrollBar
	"Do nothing"
	! !

!DiffMorph methodsFor: 'user interface' stamp: 'gvc 3/5/2010 12:43'!
initialExtent
	"Answer the initial extent for the receiver."

	^RealEstateAgent standardWindowExtent! !

!DiffMorph methodsFor: 'initialization' stamp: 'gvc 3/5/2010 12:43'!
initialize
	"Initialize the receiver."

	|exv exh opts ppCheckbox|
	super initialize.
	self prettyPrint: CodeHolder diffsWithPrettyPrint.
	ppCheckbox := self newPrettyPrintCheckboxMorph.
	opts := self newPanel
		addMorph: ((self newRow: {ppCheckbox}) 
						listCentering: #bottomRight).
	opts vResizing: #shrinkWrap.
	opts extent: opts minExtent.
	self
		srcMorph: self newSrcMorph;
		joinMorph: self newJoinMorph;
		dstMorph: self newDstMorph;
		scrollbarMorph: self newScrollbarMorph;
		mapMorph: self newMapMorph;
		changeProportionalLayout;
		addMorph: self srcMorph fullFrame: (LayoutFrame
			fractions: (0@0 corner: 0.5@1) offsets: (0@0 corner: self joinMorph width negated@opts height negated));
		addMorph: self joinMorph fullFrame: (LayoutFrame
			fractions: (0.5@0 corner: 0.5@1) offsets: (self joinMorph width negated@0 corner: 0@opts height negated));
		addMorph: self dstMorph fullFrame: (LayoutFrame
			fractions: (0.5@0 corner: 1@1) offsets: (0@0 corner: self scrollbarMorph width negated - self mapMorph width@opts height negated));
		addMorph: self scrollbarMorph fullFrame: (LayoutFrame
			fractions: (1@0 corner: 1@1) offsets: (self scrollbarMorph width negated - self mapMorph width@0 corner: self mapMorph width negated@opts height negated));
		addMorph: self mapMorph fullFrame: (LayoutFrame
			fractions: (1@0 corner: 1@1) offsets: (self mapMorph width negated@0 corner: 0@opts height negated));
		addMorph: opts fullFrame: (LayoutFrame
			fractions: (0@1 corner: 1@1) offsets: (0@opts height negated corner: 0@0)).
	exv := ExclusiveWeakMessageSend newSharedState.
	exh := ExclusiveWeakMessageSend newSharedState.
	self srcMorph
		when: #vScroll send: #srcScroll: to: self exclusive: exv;
		when: #hScroll send: #hScrollValue: to: self dstMorph exclusive: exh.
	self dstMorph
		when: #vScroll send: #dstScroll: to: self exclusive: exv;
		when: #hScroll send: #hScrollValue: to: self srcMorph  exclusive: exh.
	self
		linkSubmorphsToSplitters;
		extent: self initialExtent! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/25/2006 17:55'!
join: aJoin selected: aBoolean
	"Set the selection for the given join and update the
		src dst and join morphs."

	aJoin selected: aBoolean.
	self srcMorph changed.
	self joinMorph changed.
	self dstMorph changed! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/24/2006 12:08'!
joinColor
	"Answer the color used for the join bar."

	^Color paleBlue duller! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 10/24/2006 15:55'!
joinMappings
	"Answer the join parameters between src and dst."

	^joinMappings ifNil: [self calculateJoinMappings]! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 7/30/2009 13:42'!
joinMappings: aCollection
	"Set the join parameters between src and dst."

	joinMappings := aCollection! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 10/20/2006 10:46'!
joinMorph
	"Answer the value of joinMorph"

	^ joinMorph! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 10/20/2006 10:46'!
joinMorph: anObject
	"Set the value of joinMorph"

	joinMorph := anObject! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:12'!
joinSectionClass
	"Answer the class to use for a new join section."

	^JoinSection! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 2/10/2010 13:33'!
mapClicked: aFloat
	"Update the scrollbar value to match a click in the map."

	self scrollbarMorph
		setValue: aFloat! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 10/20/2006 10:46'!
mapMorph
	"Answer the value of mapMorph"

	^ mapMorph! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 10/20/2006 10:46'!
mapMorph: anObject
	"Set the value of mapMorph"

	mapMorph := anObject! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/26/2006 14:29'!
matchColor
	"Answer the color used to show matches."

	^Color transparent! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/24/2006 12:34'!
modificationColor
	"Answer the color used to show changes."

	^Color paleYellow alpha: 0.5! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/26/2006 13:52'!
newDstMorph
	"Answer a new dst text morph."

	^self newSrcMorph! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'BenjaminVanRyseghem 11/25/2012 04:07'!
newHighlight
	"Anewser a new highlight."

	^TextHighlightByBounds new
		color: self modificationColor;
		borderWidth: 1;
		borderColor: self edgeColor! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'BenjaminVanRyseghem 11/25/2012 04:09'!
newHighlight: type
	"Anewser a new highlight."

	^TextHighlightByBounds new
		color: (self colorForType: type);
		borderWidth: 1;
		borderColor: self edgeColor;
		fillWidth: true! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 2/3/2010 16:22'!
newJoinMorph
	"Answer a new join morph."

	|w|
	w := DiffJoinMorph splitterWidth.
	^DiffJoinMorph new
		hResizing: #shrinkWrap;
		vResizing: #spaceFill;
		extent: w@4;
		minWidth: w;
		color: self joinColor! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:11'!
newJoinSection
	"Answer a new join section."

	^self joinSectionClass new
		srcColor: self modificationColor;
		dstColor: self modificationColor;
		borderWidth: 1;
		borderColor: self edgeColor;
		addDependent: self;
		yourself! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/31/2006 14:08'!
newJoinSectionFrom: srcRange to: dstRange
	"Answer a new join section."

	|spl dpl sy1 sy2 dy1 dy2 t c|
	spl := self srcMorph textMorph paragraph lines.
	dpl := self dstMorph textMorph paragraph lines.
	t := #modification.
	sy1 := srcRange first > spl size
		ifTrue: [t := #addition.
				spl last bottom truncated - 1]
		ifFalse: [(spl at: srcRange first) top truncated - 1].
	sy2 := srcRange size < 1
		ifTrue: [t := #addition.
				 sy1 + 3]
		ifFalse: [srcRange last > spl size
				ifTrue: [spl last bottom truncated + 3]
				ifFalse: [(spl at: srcRange last) bottom truncated - 1]].
	dy1 := dstRange first > dpl size
		ifTrue: [t := #removal.
				dpl last bottom truncated - 1]
		ifFalse: [(dpl at: dstRange first) top truncated - 1].
	dy2 := dstRange size < 1
		ifTrue: [t := #removal.
				dy1 + 3]
		ifFalse: [dstRange last > dpl size
				ifTrue: [dpl last bottom truncated + 3]
				ifFalse: [(dpl at: dstRange last) bottom truncated - 1]].
	c := self colorForType: t.
	^self newJoinSection
		type: t;
		srcColor: c;
		dstColor: c;
		srcLineRange: srcRange;
		dstLineRange: dstRange;
		srcRange: (sy1 to: sy2);
		dstRange: (dy1 to: dy2);
		createHighlightsFrom: self srcMorph textMorph paragraph
		to: self dstMorph textMorph paragraph! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 2/10/2010 13:31'!
newMapMorph
	"Answer a new map morph."

	^(DiffMapMorph new
		hResizing: #shrinkWrap;
		vResizing: #spaceFill;
		extent: 20@4;
		minWidth: 20;
		borderStyle: (BorderStyle inset width: 1))
			when: #mapClicked
			send: #mapClicked:
			to: self! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 11:07'!
newMatchJoinSectionFrom: srcRange to: dstRange
	"Answer a new match join section."

	|spl dpl sy1 sy2 dy1 dy2 c|
	spl := self srcMorph textMorph paragraph lines.
	dpl := self dstMorph textMorph paragraph lines.
	sy1 := (spl at: srcRange first) top truncated.
	sy2 := (spl at: srcRange last) bottom truncated.
	dy1 := (dpl at: dstRange first) top truncated.
	dy2 := (dpl at: dstRange last) bottom truncated.
	c := self colorForType: #match.
	^self newJoinSection
		type: #match;
		borderWidth: 0;
		srcColor: c;
		dstColor: c;
		srcLineRange: srcRange;
		dstLineRange: dstRange;
		srcRange: (sy1 to: sy2);
		dstRange: (dy1 to: dy2);
		createHighlightsFrom: self srcMorph textMorph paragraph
		to: self dstMorph textMorph paragraph! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 9/2/2008 15:56'!
newPrettyPrintCheckboxMorph
	"Answer a new checkbox for specifying whether to use
	pretty printing for the diff texts."

	^self
		newCheckboxFor: self
		getSelected: #prettyPrint
		setSelected: #prettyPrint:
		getEnabled: nil
		label: 'Pretty print' translated
		help: 'If selected, pretty print will be applied to any displayed method source (eliminates trivial formatting changes)' translated! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 12/1/2008 11:48'!
newScrollbarMorph
	"Answer a new scrollbar morph."

	^ScrollBar new
		model: self;
		setValueSelector: #vScroll:;
		vResizing: #spaceFill;
		width: self theme scrollbarThickness! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 8/28/2011 12:42'!
newSrcMorph
	"Answer a new src text morph."

	^(self newTextEditorFor: self
		getText: nil
		setText: nil
		getEnabled: nil)
		hideVScrollBarIndefinitely: true;
		borderWidth: 0;
		enabled: false;
		wrapFlag: false;
		selectionColor: self textSelectionColor;
		setText: ''! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/20/2006 11:26'!
newText
	"Answer the new (dst) text."

	^self dstMorph text! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/20/2006 11:26'!
oldText
	"Answer the old (src) text."

	^self srcMorph text! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 9/2/2008 15:21'!
prettyPrint
	"Answer the value of prettyPrint"

	^ prettyPrint! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 9/2/2008 15:22'!
prettyPrint: aBoolean
	"Set the value of prettyPrint"

	prettyPrint == aBoolean ifTrue: [^self].
	prettyPrint := aBoolean.
	self updateText
	! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/24/2006 12:34'!
removalColor
	"Answer the color used to show removals."

	^Color paleRed alpha: 0.5! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 10/23/2006 15:47'!
scrollbarMorph
	"Answer the value of scrollbarMorph"

	^ scrollbarMorph! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 10/23/2006 15:47'!
scrollbarMorph: anObject
	"Set the value of scrollbarMorph"

	scrollbarMorph := anObject! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'alain.plantec 5/18/2009 15:54'!
setText
	"Set the src and dst text in the morphs applying
	prettyPrint if required."

	|src dst ctx|
	src := self srcText.
	dst := self dstText.
	ctx := self contextClass.
	(self prettyPrint and: [ctx notNil])
		ifTrue: [src isEmpty ifFalse: [
					src := ctx prettyPrinterClass 
						format: src
						in: ctx
						notifying: nil].
				dst isEmpty ifFalse: [
					dst := ctx prettyPrinterClass 
						format: dst
						in: ctx
						notifying: nil]].	
	self srcMorph setText: src; font: self theme textFont.
	self dstMorph setText: dst; font: self theme textFont! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 10/20/2006 10:46'!
srcMorph
	"Answer the value of srcMorph"

	^ srcMorph! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 10/20/2006 10:46'!
srcMorph: anObject
	"Set the value of srcMorph"

	srcMorph := anObject! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/25/2006 18:22'!
srcScroll: scrollValue
	"Called from src when scrolled by keyboard etc.."

	self scrollbarMorph value: scrollValue.
	self dstMorph vScrollBarValue: scrollValue.
	self updateJoinOffsets! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 9/2/2008 15:21'!
srcText
	"Answer the value of srcText"

	^ srcText! !

!DiffMorph methodsFor: 'accessing' stamp: 'gvc 9/2/2008 15:21'!
srcText: anObject
	"Set the value of srcText"

	srcText := anObject! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 12/21/2009 23:12'!
textSelectionColor
	"Answer the color used for thew text selection."

	^self theme settings selectionColor  alpha: 0.5! !

!DiffMorph methodsFor: '*Polymorph-Widgets' stamp: 'gvc 12/1/2008 12:27'!
themeChanged
	"Update the scrollbar width/frame."

	|offset|
	super themeChanged.
	self scrollbarMorph width: self theme scrollbarThickness.
	offset := self scrollbarMorph width negated - self mapMorph width.
	self scrollbarMorph layoutFrame leftOffset: offset.
	self dstMorph layoutFrame rightOffset: offset! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/24/2006 16:09'!
updateJoinOffsets
	"Update the src and dst offsets in the join morph
	to match the src and dst tex scroll offsets."

	self joinMorph
		srcOffset: 0 @ self srcMorph scroller offset y negated;
		dstOffset: 0 @ self dstMorph scroller offset y negated;
		changed! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 9/2/2008 15:23'!
updateText
	"Reset the text if we have some."

	(self srcText notNil and: [self dstText notNil]) ifTrue: [
		self from: self srcText to: self dstText]! !

!DiffMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/25/2006 18:22'!
vScroll: scrollValue
	"Called from standalone scroolbar.
	Scroll the srcMorph and redo the join."

	self srcMorph vScrollBarValue: scrollValue.
	self dstMorph vScrollBarValue: scrollValue.
	self updateJoinOffsets! !

DiffMorph subclass: #MergeDiffMorph
	instanceVariableNames: 'allowJoinClicks'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!MergeDiffMorph methodsFor: 'accessing' stamp: 'gvc 1/8/2009 18:17'!
allowJoinClicks
	"Answer the value of allowJoinClicks"

	^ allowJoinClicks! !

!MergeDiffMorph methodsFor: 'accessing' stamp: 'gvc 1/8/2009 18:17'!
allowJoinClicks: anObject
	"Set the value of allowJoinClicks"

	allowJoinClicks := anObject! !

!MergeDiffMorph methodsFor: 'actions' stamp: 'gvc 1/8/2009 18:17'!
calculatedJoinMappings
	"Specify click allowance for each section."
	
	^super calculatedJoinMappings do: [:j |
		j allowClick: self allowJoinClicks]! !

!MergeDiffMorph methodsFor: 'actions' stamp: 'gvc 11/1/2006 14:15'!
compositeText
	"Answer the composite text based on the selection state
	of the joins."

	^self joinMorph compositeText
	! !

!MergeDiffMorph methodsFor: 'obsolete' stamp: 'gvc 7/6/2007 15:37'!
indicateDst
	"Change the indicators of the joins to the dst side."

	self joinMappings do: [:section |
		section selectionState: #dst].
	self
		changed;
		changed: #selectedDifferences! !

!MergeDiffMorph methodsFor: 'obsolete' stamp: 'gvc 7/6/2007 15:38'!
indicateSrc
	"Change the indicators of the joins to the src side."

	self joinMappings do: [:section |
		section selectionState: #src].
	self
		changed;
		changed: #selectedDifferences! !

!MergeDiffMorph methodsFor: 'initialize-release' stamp: 'gvc 1/8/2009 18:17'!
initialize
	"Initialize the receiver."

	super initialize.
	self
		allowJoinClicks: true! !

!MergeDiffMorph methodsFor: 'actions' stamp: 'gvc 11/1/2006 14:13'!
joinSectionClass
	"Answer the class to use for a new join section."

	^MergeJoinSection! !

!MergeDiffMorph methodsFor: 'actions' stamp: 'gvc 11/1/2006 14:15'!
newJoinMorph
	"Answer a new join morph."

	^super newJoinMorph
		when: #joinClicked send: #update: to: self with: #joinClicked! !

!MergeDiffMorph methodsFor: 'actions' stamp: 'gvc 11/1/2006 14:15'!
update: aspect
	"A join has probably changed its selection state."

	super update: aspect.
	aspect == #joinClicked
		ifTrue: [self
				changed;
				changed: #selectedDifferences]! !

ComposableMorph subclass: #PSMCMergeMorph
	instanceVariableNames: 'patchMorph codeMorph merged'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!PSMCMergeMorph class methodsFor: 'as yet unclassified' stamp: 'gvc 10/30/2006 10:31'!
forMerger: aMerger
	"Answer a new instance of the receiver
	with the given merger as the model."
	
	^ self new model: aMerger! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/9/2009 15:18'!
allConflictsResolved
	"Answer whether all conflicts were resolved."
	
	^(self model ifNil: [^false]) isMerged! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/9/2009 15:43'!
cancel
	"Delete the window to cancel."

	self window delete! !

!PSMCMergeMorph methodsFor: 'accessing' stamp: 'gvc 10/29/2006 18:28'!
codeMorph
	"Answer the value of codeMorph"

	^ codeMorph! !

!PSMCMergeMorph methodsFor: 'accessing' stamp: 'gvc 10/29/2006 18:28'!
codeMorph: anObject
	"Set the value of codeMorph"

	codeMorph := anObject! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 11:53'!
compositeText
	"Answer the composite text from the patch morph."

	^self patchMorph compositeText! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/12/2009 17:12'!
compositeText: aString
	"Save the new method text for the selected method.
	Yet to be implemented."
	
	UIManager default inform: 'Saving of replacement methods is not yet imlemented.'! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/29/2009 12:52'!
conflictCount
	"Answer the number of conflicts that are unresolved."

	^(self model ifNil: [^0]) conflicts count: [:c | c isResolved not]! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/29/2009 12:50'!
conflictCountString
	"Answer a string describing the number of conflicts."

	|count|
	count := self conflictCount.
	^count = 1
		ifTrue: ['1 conflict' translated]
		ifFalse: ['{1} conflicts' translated format: {count}]! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/30/2006 09:47'!
defaultTitle
	"Answer the default title label for the receiver."

	^'Merge' translated! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 2/3/2010 16:05'!
fromDescription: aString
	"Set the description for the left-hand side of the merge,
	typically 'currently in image'."

	self patchMorph fromDescription: aString! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 2/4/2010 16:49'!
initialColorInSystemWindow: aSystemWindow
	"Answer the colour the receiver should be when added to a SystemWindow."

	^Color transparent! !

!PSMCMergeMorph methodsFor: 'initialization' stamp: 'gvc 1/9/2009 15:38'!
initialize
	"Initialize the receiver."

	|buttons buttonsHeight|
	super initialize.
	buttons := self newButtonsMorph.
	buttonsHeight := buttons minExtent y.
	self
		merged: false;
		patchMorph: self newPatchMorph;
		codeMorph: self newCodeMorph;
		changeProportionalLayout;
		addMorph: self patchMorph
		fullFrame: (LayoutFrame fractions: (0@0 corner: 1@0.6));
		addMorph: self codeMorph
		fullFrame: (LayoutFrame fractions: (0@0.6 corner: 1@1) offsets: (0@0 corner: 0@buttonsHeight negated));
		addMorph: self newButtonsMorph
		fullFrame: (LayoutFrame fractions: (0@1 corner: 1@1) offsets: (0@buttonsHeight negated corner: 0@0));
		addPaneSplitters! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/9/2009 15:44'!
merge
	"Do the merge and close if no conflicts."
	
	^self model isMerged
		ifTrue: [self merged: true.
				self window delete]
		ifFalse: [self inform: 'You must resolve all conflicts first.' translated]! !

!PSMCMergeMorph methodsFor: 'accessing' stamp: 'gvc 1/8/2009 17:42'!
merged
	"Answer the value of merged. Indicates whether
	the merge button was pressed with no conflicts remaining."

	^ merged! !

!PSMCMergeMorph methodsFor: 'accessing' stamp: 'gvc 1/8/2009 17:42'!
merged: anObject
	"Set the value of merged"

	merged := anObject! !

!PSMCMergeMorph methodsFor: 'initialization' stamp: 'StephaneDucasse 6/24/2011 15:08'!
model: aMerger
	"Set the model and update the window."

	|grouped sorted|
	super model: aMerger.
	grouped := aMerger operations
		groupBy: [:o | o targetClassName ifNil: [o shortSummary]]
		having: [:g | true].
	sorted := OrderedCollection new.
	grouped keys asSortedCollection do: [:k |
		sorted addAll: ((grouped at: k) asSortedCollection: [:a :b | a shortSummary <= b shortSummary])].		
	self patchMorph model: sorted! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/29/2009 12:43'!
newButtonsMorph
	"Answer a new buttons morph."

	^(self newRow: {
			self newConflictsButton.
			self newToolSpacer hResizing: #spaceFill.
			self newMergeButton.
			self newCancelButton})
		removeProperty: #fillStyle;
		listCentering: #bottomRight;
		layoutInset: 4! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 11:53'!
newCodeMorph
	"Answer a new code morph."

	^(self newTextEditorFor: self
		getText: #compositeText
		setText: #compositeText:
		getEnabled: nil)
		wrapFlag: false;
		setText: ''! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/29/2009 12:54'!
newConflictsButton
	"Answer a new button for displaying the count
	of outstanding conflicts and navigating to each.
	It will be disabled if all conflicts are resolved."

	^(self
		newButtonFor: self
		getState: nil
		action:  #selectNextConflict
		arguments: #()
		getEnabled: #notAllConflictsResolved
		getLabel:  #conflictCountString
		help: 'Select the next conflict in the tree' translated)
		vResizing: #spaceFill "workaround until table  layout fixed"! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/29/2009 12:42'!
newMergeButton
	"Answer a new button for performing the merge.
	It will be disabled if any conflicts are unresolved."

	^self
		newButtonFor: self
		action:  #merge
		getEnabled:  #allConflictsResolved
		label:  'Merge' translated
		help: 'Merge the version into the image' translated! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:26'!
newPatchMorph
	"Answer a new patch morph."

	^PSMCMergePatchMorph new
		borderWidth: 0;
		addDependent: self;
		yourself! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/29/2009 12:46'!
notAllConflictsResolved
	"Answer whether any conflicts are unresolved."
	
	^self allConflictsResolved not! !

!PSMCMergeMorph methodsFor: 'accessing' stamp: 'gvc 10/29/2006 18:28'!
patchMorph
	"Answer the value of patchMorph"

	^ patchMorph! !

!PSMCMergeMorph methodsFor: 'accessing' stamp: 'gvc 10/29/2006 18:28'!
patchMorph: anObject
	"Set the value of patchMorph"

	patchMorph := anObject! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 1/29/2009 12:57'!
selectNextConflict
	"Select the next conflict in the tree."
	
	self patchMorph selectNextConflict! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 2/3/2010 16:05'!
toDescription: aString
	"Set the description for the right-hand side of the merge,
	typically 'incoming'."

	self patchMorph toDescription: aString! !

!PSMCMergeMorph methodsFor: 'updating' stamp: 'gvc 1/29/2009 13:22'!
update: aspect
	"A join has probably changed its selection state."

	super update: aspect.
	aspect == #compositeText
		ifTrue: [self updateCode].
	aspect == #changes
		ifTrue: [self
				changed: #conflictCountString;
				changed: #allConflictsResolved;
				changed: #notAllConflictsResolved]! !

!PSMCMergeMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 11:54'!
updateCode
	"Update the code morph to match selected differences."

	self changed: #compositeText! !

ListItemWrapper subclass: #PSMCChangeWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!PSMCChangeWrapper methodsFor: 'accessing' stamp: 'gvc 1/14/2009 12:42'!
actualClass
	"Answer the class represented in the receiver."

	^(self operation ifNil: [^nil]) targetClass! !

!PSMCChangeWrapper methodsFor: 'accessing' stamp: 'AlainPlantec 10/19/2010 21:08'!
icon
	"Answer a form with an icon to represent the receiver"
	|o|
	o := self operation.
	o isNil ifTrue: [^self theme smallJumpIcon].
	o isAddition ifTrue: [^self theme smallOkIcon].
	o isRemoval ifTrue: [^self theme smallCancelIcon].
	^self theme smallForwardIcon! !

!PSMCChangeWrapper methodsFor: 'testing' stamp: 'gvc 7/6/2007 14:41'!
isConflict
	"Answer whether the receiver is a conflict item."

	^false! !

!PSMCChangeWrapper methodsFor: 'accessing' stamp: 'gvc 7/30/2009 13:44'!
model: anObject
	"Set the model."

	model := anObject! !

!PSMCChangeWrapper methodsFor: 'accessing' stamp: 'gvc 10/30/2006 11:22'!
operation
	"Answer the underlying operation."

	self subclassResponsibility ! !

!PSMCChangeWrapper methodsFor: 'accessing' stamp: 'IgorStasenko 6/24/2011 17:29'!
preferredColor

	| op |
	op := self operation.
	op ifNil: [ ^ nil ].
	(op isConflict and: [ op isResolved not ] ) ifTrue: [  ^ Color red ].
	op remoteChosen ifFalse: [ ^ Color gray ].
	^ nil "use default color "! !

!PSMCChangeWrapper methodsFor: 'accessing' stamp: 'AlainPlantec 10/19/2010 21:09'!
theme
	^ UITheme current! !

PSMCChangeWrapper subclass: #PSMCClassChangeWrapper
	instanceVariableNames: 'conflict contents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!PSMCClassChangeWrapper methodsFor: 'accessing' stamp: 'lr 3/14/2010 21:13'!
actualClass
	"Answer the class represented in the receiver."

	^ super actualClass ifNil: [ Smalltalk globals classNamed: self item ]! !

!PSMCClassChangeWrapper methodsFor: 'choosing' stamp: 'StephaneDucasse 6/24/2011 16:22'!
chooseLocal
	"Choose the local version."

	self operation chooseLocal! !

!PSMCClassChangeWrapper methodsFor: 'choosing' stamp: 'StephaneDucasse 6/24/2011 16:22'!
chooseRemote
	"Choose the remote version."

	self operation chooseRemote! !

!PSMCClassChangeWrapper methodsFor: 'choosing' stamp: 'StephaneDucasse 6/24/2011 16:22'!
clearChoice
	"Choose neither version (be in conflict)."

	self operation clearChoice! !

!PSMCClassChangeWrapper methodsFor: 'accessing' stamp: 'gvc 4/1/2009 13:30'!
contents
	"Answer the contents of the change."

	^contents ifNil: [contents := self gatherContents]! !

!PSMCClassChangeWrapper methodsFor: 'accessing' stamp: 'gvc 4/1/2009 13:30'!
gatherContents
	"Answer the contents of the change from the model."

	^((self model select: [:i | i targetClassName = self item and: [
			i definition isClassDefinition not]])
		collect: [:o |  o patchWrapper model: self model]) asSortedCollection: [:a :b |
			a asString <= b asString]! !

!PSMCClassChangeWrapper methodsFor: 'accessing' stamp: 'StephaneDucasse 6/24/2011 16:20'!
icon
	"Answer a form with an icon to represent the receiver"
	
	self isConflict ifTrue: [
		self localChosen ifTrue: [^self theme smallBackIcon]].
	^super icon! !

!PSMCClassChangeWrapper methodsFor: 'testing' stamp: 'StephaneDucasse 6/24/2011 16:18'!
isConflict
	"Answer whether the receiver is a conflict item."

	| op |
	op := self operation.
	^ op notNil and: [op isConflict ]! !

!PSMCClassChangeWrapper methodsFor: 'testing' stamp: 'StephaneDucasse 6/24/2011 16:18'!
localChosen
	"Answer whether the local version is chosen."

	^self operation localChosen! !

!PSMCClassChangeWrapper methodsFor: 'accessing' stamp: 'StephaneDucasse 6/24/2011 16:17'!
operation
	"Answer the patch operation for the class itself or nil if none."

	|o|
	o := self model detect: [:i | i targetClassName = self item and: [
				i definition isClassDefinition]] ifNone: [ ^ nil ].

	^o! !

!PSMCClassChangeWrapper methodsFor: 'testing' stamp: 'gvc 7/6/2007 15:42'!
remoteChosen
	"Answer whether the remote version is chosen."

	^self conflict remoteChosen! !

PSMCChangeWrapper subclass: #PSMCPatchOperationWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

PSMCPatchOperationWrapper subclass: #PSMCOrganizationChangeWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!PSMCPatchOperationWrapper methodsFor: 'converting' stamp: 'GaryChambers 10/26/2011 16:19'!
asString
	"Answer the method name."
	
	^self item definition summary! !

!PSMCPatchOperationWrapper methodsFor: 'accessing' stamp: 'StephaneDucasse 6/24/2011 15:52'!
chooseLocal
	"Choose the local version."

	self operation chooseLocal! !

!PSMCPatchOperationWrapper methodsFor: 'accessing' stamp: 'StephaneDucasse 6/24/2011 15:52'!
chooseRemote
	"Choose the remote version."

	self operation chooseRemote! !

!PSMCPatchOperationWrapper methodsFor: 'testing' stamp: 'StephaneDucasse 6/24/2011 15:51'!
clearChoice
	"Choose neither version (be in conflict)."

	self operation clearChoice! !

!PSMCPatchOperationWrapper methodsFor: 'accessing' stamp: 'StephaneDucasse 6/24/2011 15:47'!
icon
	"Answer a form with an icon to represent the receiver"
	
	self item localChosen ifTrue: [^self theme smallBackIcon].
	^super icon! !

!PSMCPatchOperationWrapper methodsFor: 'testing' stamp: 'StephaneDucasse 6/24/2011 15:47'!
isConflict
	"Answer whether the receiver is a conflict item."

	^ self operation isConflict! !

!PSMCPatchOperationWrapper methodsFor: 'testing' stamp: 'StephaneDucasse 6/24/2011 15:48'!
localChosen
	"Answer whether the local version is chosen."

	^self operation localChosen! !

!PSMCPatchOperationWrapper methodsFor: 'accessing' stamp: 'gvc 10/26/2006 12:00'!
operation
	"Answer the pacth operation for the receiver or nil if none."

	^self item! !

!PSMCPatchOperationWrapper methodsFor: 'testing' stamp: 'StephaneDucasse 6/24/2011 15:51'!
remoteChosen
	"Answer whether the remote version is chosen."

	^self operation remoteChosen! !

TextDiffBuilder subclass: #InlineTextDiffBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!InlineTextDiffBuilder methodsFor: 'as yet unclassified' stamp: 'gvc 9/2/2008 16:10'!
split: aString
	"Answer the split 'lines' by splitting on whitespace."
	
	|str lines sep|
	lines := OrderedCollection new.
	sep := Character separators, '^()[]{}''"`;.'.
	str := aString readStream.
	[str atEnd] whileFalse: [
		lines add: (str upToAny: sep).
		str atEnd ifFalse: [
			str skip: -1.
			lines add: str next asString]].
	^lines! !

!MCVersionHistoryBrowser methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 2/3/2010 15:31'!
viewChanges
	"View the changes between a prior version and this version."
	
	self
		viewChanges: (self baseSnapshot patchRelativeToBase: self selectedSnapshot)
		from:  self selectedInfo name
		to: ancestry name! !

!MCVersionHistoryBrowser methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 2/9/2010 14:06'!
viewChanges: patch from: fromDescription to: toDescription
	"Open a browser on the patch."
	
	|patchLabel|
	patchLabel := 'Changes between {1} and {2}' translated format: {fromDescription. toDescription}.
	PSMCPatchMorph usedByDefault
		ifTrue: [((PSMCPatchMorph forPatch: patch)
						fromDescription: fromDescription;
						toDescription: toDescription;
						newWindow)
					title: patchLabel;
					open]
		ifFalse: [(MCPatchBrowser forPatch: patch)
					label: patchLabel;
					show]! !

ProportionalSplitterMorph subclass: #DiffJoinMorph
	instanceVariableNames: 'srcOffset dstOffset mappings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!DiffJoinMorph class methodsFor: 'as yet unclassified' stamp: 'gvc 2/3/2010 16:20'!
splitterWidth
	"Answer the width of splitter to use for dif joins."
	
	^30! !

!DiffJoinMorph methodsFor: 'as yet unclassified' stamp: 'gvc 7/30/2009 13:43'!
compositeText
	"Answer the composite text based on the selection state
	of the joins."

	|t|
	t := Text new.
	self mappings do: [:j |
		j appendToCompositeText: t].
	^t! !

!DiffJoinMorph methodsFor: 'initialize' stamp: 'gvc 10/20/2006 14:21'!
defaultColor
	"Answer the default color for the receiver."

	^Color transparent! !

!DiffJoinMorph methodsFor: 'drawing' stamp: 'gvc 10/31/2006 12:08'!
drawOn: aCanvas
	"Draw the indicators for the mappings."
	
	super drawOn: aCanvas.
	aCanvas translateBy: self topLeft clippingTo: self clippingBounds during: [:c |
		self mappings do: [:j | j drawOn: c]]! !

!DiffJoinMorph methodsFor: 'accessing' stamp: 'gvc 10/24/2006 16:02'!
dstOffset
	"Answer the value of dstOffset"

	^ dstOffset! !

!DiffJoinMorph methodsFor: 'accessing' stamp: 'gvc 10/31/2006 11:33'!
dstOffset: anInteger
	"Set the dstOffset."

	dstOffset := anInteger.
	self mappings do: [:j |
		j dstOffset: anInteger]! !

!DiffJoinMorph methodsFor: 'geometry' stamp: 'gvc 10/31/2006 13:18'!
extent: aPoint
	"Update the shapes of the joins."
	
	super extent: aPoint.
	self updateMappings! !

!DiffJoinMorph methodsFor: 'initialize' stamp: 'gvc 10/31/2006 11:44'!
initialize
	"Initialize the receiver."

	super initialize.
	self
		mappings: OrderedCollection new;
		srcOffset: 0@0;
		dstOffset: 0@0! !

!DiffJoinMorph methodsFor: 'layout' stamp: 'gvc 10/31/2006 13:18'!
layoutBounds: aRectangle
	"Set the bounds for laying out children of the receiver."
	
	super layoutBounds: aRectangle.
	self updateMappings! !

!DiffJoinMorph methodsFor: 'accessing' stamp: 'gvc 10/20/2006 11:50'!
mappings
	"Answer the value of mappings"

	^ mappings! !

!DiffJoinMorph methodsFor: 'accessing' stamp: 'gvc 10/31/2006 13:19'!
mappings: anObject
	"Set the value of mappings"

	mappings := anObject.
	self updateMappings.
	self changed! !

!DiffJoinMorph methodsFor: 'event handling' stamp: 'gvc 11/1/2006 12:00'!
mouseDown: evt
	"Check for a click."

	|cj|
	cj := self mappings
		detect: [:j | j containsPoint: evt position - self topLeft]
		ifNone: [].
	cj ifNotNil: [
		cj clicked.
		self triggerEvent: #joinClicked].
	super mouseDown: evt! !

!DiffJoinMorph methodsFor: 'accessing' stamp: 'gvc 10/24/2006 16:02'!
srcOffset
	"Answer the value of srcOffset"

	^ srcOffset! !

!DiffJoinMorph methodsFor: 'accessing' stamp: 'gvc 10/31/2006 11:33'!
srcOffset: anInteger
	"Set the srcOffset."

	srcOffset := anInteger.
	self mappings do: [:j |
		j srcOffset: anInteger]! !

!DiffJoinMorph methodsFor: 'as yet unclassified' stamp: 'gvc 10/31/2006 13:18'!
updateMappings
	"Update the shapes of the joins."
	
	self mappings do: [:j |
		j width: self width]! !

!MCDefinition methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 12/12/2008 16:18'!
actualClass
	"Since the targetClass call on a patch operation will fail
	otherwise."
	
	^nil! !

!MCDefinition methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 4/1/2009 12:11'!
className
	"Answer the class name here or nil if not applicable."
	
	^nil! !

!MCDefinition methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 4/1/2009 12:14'!
fullClassName
	"Answer the className by default."
	
	^self className! !

BorderedMorph subclass: #DiffMapMorph
	instanceVariableNames: 'mappings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!DiffMapMorph methodsFor: 'accessing' stamp: 'gvc 10/26/2006 15:11'!
adoptPaneColor: paneColor
	"Change our border color too."
	
	|c|
	super adoptPaneColor: paneColor.
	paneColor ifNil: [^self].
	c := paneColor alphaMixed: 0.1 with: Color white.
	self fillStyle: ((GradientFillStyle
			ramp: (self gradientRampForColor: c))
		origin: self bounds topLeft;
		direction: 0@ self height).
	self borderStyle baseColor: paneColor! !

!DiffMapMorph methodsFor: 'accessing' stamp: 'gvc 10/26/2006 14:10'!
defaultColor
	"Answer the default color for the receiver."

	^Color white! !

!DiffMapMorph methodsFor: 'nil' stamp: 'gvc 10/26/2006 15:05'!
drawOn: aCanvas
	"Draw the indicators for the mappings."
	
	|b f|
	b := self innerBounds insetBy: 2.
	super drawOn: aCanvas.
	b height < 1 ifTrue: [^self].
	f := self mappingsHeight.
	f < 1 ifTrue: [^self].
	f := b height / f.
	aCanvas clipBy: self clippingBounds during: [:c |
		self mappings do: [:j |
			j drawMapOn: c in: b scale: f]]! !

!DiffMapMorph methodsFor: 'accessing' stamp: 'gvc 3/21/2008 17:14'!
extent: newExtent
	"Update the gradient."
	
	super extent: newExtent.
	(self fillStyle notNil and: [self fillStyle isOrientedFill])
		ifTrue: [self fillStyle direction: 0@self height]! !

!DiffMapMorph methodsFor: 'accessing' stamp: 'gvc 10/26/2006 15:12'!
gradientRampForColor: c
	"Answer the background gradient ramp to use."
	
	^{0.0->c darker duller. 0.1-> c lighter.
		0.9->c twiceLighter. 1.0->c darker}! !

!DiffMapMorph methodsFor: 'as yet unclassified' stamp: 'gvc 2/10/2010 13:30'!
handlesMouseDown: anEvent
	"Answer true to report mouse down activity."
	
	^true! !

!DiffMapMorph methodsFor: 'accessing' stamp: 'gvc 10/26/2006 14:08'!
initialize
	"Initialize the receiver."

	super initialize.
	self
		mappings: #()! !

!DiffMapMorph methodsFor: 'accessing' stamp: 'gvc 10/26/2006 13:57'!
mappings
	"Answer the value of mappings"

	^ mappings! !

!DiffMapMorph methodsFor: 'accessing' stamp: 'gvc 10/26/2006 14:15'!
mappings: anObject
	"Set the value of mappings"

	mappings := anObject.
	self changed! !

!DiffMapMorph methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 10:51'!
mappingsHeight
	"Answer the maximum y of all the mappings."
	
	self mappings ifEmpty: [^0].
	^self mappings last dst range last ! !

!DiffMapMorph methodsFor: 'as yet unclassified' stamp: 'gvc 2/10/2010 13:28'!
mouseDown: event
	"Trigger a 0..1 range based on location."
	
	|b|
	b := self innerBounds insetBy: 2.
	self
		triggerEvent: #mapClicked
		with: (((event cursorPoint y asFloat - b top / b height) 
				min: 1) max: 0)! !

!MCPatchBrowser methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 10/24/2006 15:44'!
diffSelection
	"Open a diff browser on the selection."
	
	selection ifNotNil:
		[selection diff]! !

!MCPatchBrowser methodsFor: '*Polymorph-Tools-Diff' stamp: 'StephaneDucasse 2/10/2012 15:50'!
methodListMenu: aMenu

	selection ifNotNil:
		[aMenu addList:#(
			('Install'	 installSelection)
			('Diff'	diffSelection)
			('Revert'	 revertSelection)
			-)].
	super methodListMenu: aMenu.
	^ aMenu
! !

!MCModification methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 4/2/2009 13:26'!
diff
	"Open a diff browser on the changes."

	(DiffMorph
		from: self fromSource
		to: self toSource
		contextClass: (self isClassPatch ifTrue: [nil] ifFalse: [self targetClass]))
			extent: 400@300;
			openInWindowLabeled: 'Diff' translated! !

!MCModification methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 1/14/2009 14:11'!
diffFromSource
	"Answer fromSource of the modification. If a class patch then answer
	the fromSource with the class-side definition and comment appended."
	
	^self isClassPatch
		ifTrue: [self fromSource, String cr, String cr,
				obsoletion classDefinitionString, String cr, String cr,
				obsoletion commentStamp, String cr,
				obsoletion comment]
		ifFalse: [self fromSource]! !

!MCModification methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 1/14/2009 14:12'!
diffToSource
	"Answer toSource of the modification. If a class patch then answer
	the toSource with the class-side definition and comment appended."
	
	^self isClassPatch
		ifTrue: [self toSource, String cr, String cr, 
				modification classDefinitionString, String cr, String cr,
				modification commentStamp, String cr,
				modification comment]
		ifFalse: [self toSource]! !

!MCMergeBrowser methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 5/16/2007 10:40'!
chooseAllNewerConflicts
	"Notify the potential new state of canMerge."
	
	conflicts do: [ :ea | ea chooseNewer ].
	self changed: #text; changed: #list; changed: #canMerge! !

!MCMergeBrowser methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 5/16/2007 10:41'!
chooseAllOlderConflicts
	"Notify the potential new state of canMerge."
	
	conflicts do: [ :ea | ea chooseOlder ].
	self changed: #text; changed: #list; changed: #canMerge! !

!MCMergeBrowser methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 5/16/2007 10:41'!
chooseAllUnchosenLocal
	"Notify the potential new state of canMerge."
	
	conflicts do: [ :ea | ea isResolved ifFalse: [ ea chooseLocal ] ].
	self changed: #text; changed: #list; changed: #canMerge! !

!MCMergeBrowser methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 5/16/2007 10:41'!
chooseAllUnchosenRemote
	"Notify the potential new state of canMerge."
	
	conflicts do: [ :ea | ea isResolved ifFalse: [ ea chooseRemote ] ].
	self changed: #text; changed: #list; changed: #canMerge! !

!MCMergeBrowser methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 5/16/2007 10:41'!
chooseLocal
	"Notify the potential new state of canMerge."
	
	self conflictSelectionDo:
		[selection chooseLocal.
		self changed: #text; changed: #list; changed: #canMerge]! !

!MCMergeBrowser methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 5/16/2007 10:41'!
chooseRemote
	"Notify the potential new state of canMerge."
	
	self conflictSelectionDo:
		[selection chooseRemote.
		self changed: #text; changed: #list; changed: #canMerge]! !

!MCMergeBrowser methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 4/2/2009 13:14'!
selection: aNumber
	"Notify change of conflicts too."
	
	super selection: aNumber.
	self changed: #selectionIsConflicted! !

!MCPatchOperation methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 10/24/2006 15:40'!
diff
	"Open a diff browser on the changes."
! !

!MCPatchOperation methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 1/14/2009 14:07'!
diffFromSource
	"Answer fromSource of the operation for a diff tool."
	
	^self fromSource! !

!MCPatchOperation methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 1/14/2009 14:07'!
diffToSource
	"Answer toSource of the operation for a diff tool."
	
	^self toSource! !

!MCPatchOperation methodsFor: '*Polymorph-Tools-Diff' stamp: 'nice 3/29/2011 00:11'!
shortSummary
	"Answer a short summary of the receiver."

	^self shortSummaryPrefix , self shortSummarySuffix! !

!MCPatchOperation methodsFor: '*Polymorph-Tools-Diff' stamp: 'BenjaminVanRyseghem 9/28/2011 15:25'!
shortSummaryPrefix
	"Answer a short summary prefix of the receiver."

	^ self definition summary! !

!MCPatchOperation methodsFor: '*Polymorph-Tools-Diff' stamp: 'nice 3/29/2011 00:11'!
shortSummarySuffix
	"Answer a short summary suffix of the receiver."

	^self fromSource = self toSource
		ifTrue: [' (revision changed)']
		ifFalse: ['']! !

!MCPatchOperation methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 4/1/2009 12:15'!
targetClassName
	"Answer the full class *name* of the target since the class may no longer exist."
	
	^self definition fullClassName! !

!MCVersionInspector methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 2/9/2010 14:06'!
viewChanges: patch from: fromDescription to: toDescription
	"Open a patch morph for the changes."
	
	|title|
	title := 'Changes from {1} to {2}' format: {fromDescription. toDescription}.
	PSMCPatchMorph usedByDefault
		ifTrue: [((PSMCPatchMorph forPatch: patch)
						fromDescription: fromDescription;
						toDescription: toDescription;
						newWindow)
					title: title;
					open]
		ifFalse: [(MCPatchBrowser forPatch: self version changes)
					showLabelled: title]! !

Object subclass: #JoinSection
	instanceVariableNames: 'src dst borderWidth borderColor type width shape'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!JoinSection methodsFor: 'as yet unclassified' stamp: 'BenjaminVanRyseghem 11/25/2012 04:08'!
addHighlightsFrom: srcBlock to: dstBlock to: aCollection color: aColor
	"Add the highlights required for the given character blocks
	of a paragraph. May be up to three highlights depending
	on the line spans."
	
	srcBlock textLine = dstBlock textLine
		ifTrue: [aCollection add: (TextHighlightByBounds new
				color: aColor;
				bounds: (srcBlock topLeft corner: dstBlock bottomRight))]
		ifFalse: [aCollection
					add: (TextHighlightByBounds new
							color: aColor;
							bounds: (srcBlock topLeft corner: srcBlock textLine bottomRight));
					add: (TextHighlightByBounds new
							fillWidth: true;
							color: aColor;
							bounds: (srcBlock bottomLeft corner: dstBlock topRight));
					add: (TextHighlightByBounds new
							color: aColor;
							bounds: (dstBlock textLine topLeft corner: dstBlock bottomRight))]! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:16'!
borderColor
	"Answer the value of borderColor"

	^ borderColor! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:16'!
borderColor: aColor
	"Set the value of borderColor"

	borderColor := aColor.
	self updateHighlights! !

!JoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:22'!
borderColorToUse
	"Answer the border color to use."
	
	^self borderColor! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:16'!
borderWidth
	"Answer the value of borderWidth"

	^ borderWidth! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:16'!
borderWidth: anInteger
	"Set the value of borderWidth"

	borderWidth := anInteger.
	self src highlight notNil ifTrue: [
		self src highlight borderWidth: anInteger].
	self dst highlight notNil ifTrue: [
		self dst highlight borderWidth: anInteger]! !

!JoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:17'!
clicked
	"The receiver or a highlight was clicked."

	self wantsClick ifFalse: [^false].
	^true! !

!JoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:18'!
containsPoint: aPoint
	"Answer whether the receiver contains the given point."

	^self shape containsPoint: aPoint! !

!JoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:18'!
createHighlights
	"Create and store the src and dst highlights."

	|s d|
	s := OrderedCollection new.
	d := OrderedCollection new.
	s add: (self newHighlight
			color: self src color;
			borderWidth: self borderWidth;
			bounds: (0@self src range first corner: 0@(self src range last + 1));
			borderSides: #(top left bottom)).
	d add: (self newHighlight
			color: self dst color;
			borderWidth: self borderWidth;
			bounds: (0@self dst range first corner: 0@(self dst range last + 1));
			borderSides: #(top right bottom)).
	self src highlights: s.
	self dst highlights: d! !

!JoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:18'!
createHighlightsFrom: srcPara to: dstPara
	"Create and store the src and dst highlights.
	Use the given paragraphs to determine inline
	diffs."

	|s d si di  srcText dstText diffs i sb eb line|
	self createHighlights.
	self src lineRange notEmpty
		ifTrue: [ line := srcPara lines at: self src lineRange first.
				si := line first.
				line := srcPara lines at: self src lineRange last.
				srcText := srcPara string copyFrom: si to: line last]
		ifFalse: [srcText := ''].
	self dst lineRange notEmpty
		ifTrue: [line := dstPara lines at: self dst lineRange first.
				di := line first.
				line := dstPara lines at: self dst lineRange last.
				dstText := dstPara string copyFrom: di to: line last]
		ifFalse: [dstText := ''].
	self src text: srcText.
	self dst text: dstText.
	self type = #modification ifFalse: [^self].
	s := self src highlights.
	d := self dst highlights.
	diffs := (InlineTextDiffBuilder from: srcText to: dstText)
		buildPatchSequence
			aggregateRuns: [:e | e key].
	diffs do: [:c |
		c first key = #match
			ifTrue: [c do: [:a |
					si := si + a value size.
					di := di + a value size]].
		c first key = #insert
			ifTrue: [i := di.
					c do: [:a | di := di + a value size].
					sb := dstPara characterBlockForIndex: i.
					eb := dstPara characterBlockForIndex: di - 1.
					self addHighlightsFrom: sb to: eb to: d color: (Color green alpha: 0.3)].
		c first key = #remove
			ifTrue: [i := si.
					c do: [:a | si := si + a value size].
					sb := srcPara characterBlockForIndex: i.
					eb := srcPara characterBlockForIndex: si - 1.
					self addHighlightsFrom: sb to: eb to: s color: (Color red alpha: 0.3)]]! !

!JoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:24'!
drawMapOn: aCanvas in: rect scale: scale
	"Draw the join on the given canvas scaled into the given rectangle."
	
	self type = #match ifTrue: [^self].
	aCanvas
		frameAndFillRectangle: (rect left @ (((self dst range first max: 0) * scale) truncated + rect top)
						corner: (rect right @ ((self dst range last * scale) truncated + rect top)))
		fillColor: (self fillStyleFor: rect)
		borderWidth: 1
		borderColor: self borderColorToUse! !

!JoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:23'!
drawOn: aCanvas
	"Draw the join on the given canvas."
	
	|v bc|
	(self src color isTransparent and: [self dst color isTransparent])
		ifTrue: [^self].
	v := self shape vertices.
	aCanvas
		drawPolygon: v
		fillStyle: (self fillStyleFor: self shape bounds).
	(self borderWidth > 0 and: [self borderColor isTransparent not]) ifTrue: [
		bc := self borderColorToUse.
		aCanvas
			line: v first + (0@self borderWidth // 2)
			to: v second + (-1@self borderWidth // 2)
			width: self borderWidth
			color: bc;
			line: v third - (1@(self borderWidth // 2))
			to: v fourth - (0@(self borderWidth // 2))
			width: self borderWidth
			color: bc]! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
dst
	"Answer the value of dst"

	^ dst! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
dst: anObject
	"Set the value of dst"

	dst := anObject! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
dstColor: aColor
	"Set the dst color"

	self dst color: aColor! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
dstLineRange: anInterval
	"Set the dst lineRange."

	self dst lineRange: anInterval! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
dstOffset: aPoint
	"Set the dst offset."

	self dst offset:  aPoint.
	self updateShape! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
dstRange: anInterval
	"Set the dst range."

	self dst range: anInterval.
	self updateShape! !

!JoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:19'!
fillStyleFor: rect
	"Answer the fillStyle to use for the given rectangle."
	
	^self src color = self dst color
		ifTrue: [self src color]
		ifFalse: [(GradientFillStyle ramp: {0.0 -> self src color. 1.0 -> self dst color})
				direction: rect width@0;
				origin: rect topLeft]! !

!JoinSection methodsFor: 'initialize-release' stamp: 'gvc 11/1/2006 14:20'!
initialize
	"Initialize the receiver."

	super initialize.
	self
		src: JoinSide new;
		dst: JoinSide new;
		shape: Polygon new;
		width: 0;
		borderWidth: 0;
		borderColor: Color transparent;
		type: #modification! !

!JoinSection methodsFor: 'as yet unclassified' stamp: 'BenjaminVanRyseghem 11/25/2012 04:07'!
newHighlight
	"Anwser a new highlight."

	^TextHighlightByBounds new
		borderWidth: 1;
		borderColor: self borderColor;
		fillWidth: true! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
shape
	"Answer the value of shape"

	^ shape! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
shape: anObject
	"Set the value of shape"

	shape := anObject! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
src
	"Answer the value of src"

	^ src! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
src: anObject
	"Set the value of src"

	src := anObject! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
srcColor: aColor
	"Set the src color."

	self src color: aColor! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
srcLineRange: anInterval
	"Set the src lneRange."

	self src lineRange: anInterval! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
srcOffset: aPoint
	"Set the src offset"

	self src offset:  aPoint.
	self updateShape! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
srcRange: anInterval
	"Set the  src range"

	self src range: anInterval.
	self updateShape! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
type
	"Answer the value of type"

	^ type! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:19'!
type: anObject
	"Set the value of type"

	type := anObject! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:28'!
updateHighlights
	"Update the highlight border colors."

	|bc|
	(self src isNil or: [self dst isNil]) ifTrue: [^self].
	bc := self borderColorToUse.
	self src highlight notNil ifTrue: [
		self src highlight borderColor: bc].
	self dst highlight notNil ifTrue: [
		self dst highlight borderColor: bc]! !

!JoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:19'!
updateShape
	"Update the receiver's shape."

	(self src range isNil or: [self dst range isNil]) ifTrue: [^self].
	self shape: (Polygon vertices:
		{(0@ self src range first) + self src offset. (self width @ self dst range first) + self dst offset.
		(self width @ self dst range last) + self dst offset. (0@self src range last) + self src offset})! !

!JoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:19'!
wantsClick
	"Don't if we are transparent for now."

	^(self src color isTransparent and: [self dst color isTransparent]) not! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:21'!
width
	"Answer the value of width"

	^ width! !

!JoinSection methodsFor: 'accessing' stamp: 'gvc 11/1/2006 14:21'!
width: anObject
	"Set the value of width"

	width := anObject.
	self updateShape! !

JoinSection subclass: #MergeJoinSection
	instanceVariableNames: 'selectedBorderColor selected selectionState stateIcons allowClick'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!MergeJoinSection methodsFor: 'accessing' stamp: 'gvc 1/8/2009 18:04'!
allowClick
	"Answer the value of allowClick"

	^ allowClick! !

!MergeJoinSection methodsFor: 'accessing' stamp: 'gvc 1/8/2009 18:04'!
allowClick: anObject
	"Set the value of allowClick"

	allowClick := anObject! !

!MergeJoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 12:10'!
appendToCompositeText: aText
	"If appropriate append the relevant src or dst text to the given text."
	
	self selectionState == #src ifTrue: [^aText append: self src text].
	self selectionState == #dst ifTrue: [^aText append: self dst text].
	self selectionState == #both ifTrue: [
		^aText
			append: self src text;
			append: self dst text]! !

!MergeJoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:22'!
borderColorToUse
	"Answer the border color to use based on the selection state."
	
	^self selected
		ifTrue: [self selectedBorderColor]
		ifFalse: [super borderColorToUse]! !

!MergeJoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 10/31/2006 13:09'!
clicked
	"The receiver or a highlight was clicked."

	self wantsClick ifFalse: [^false].
	self selectNextState.
	^true! !

!MergeJoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 14:18'!
containsPoint: aPoint
	"Answer whether the receiver contains the given point."

	^(super containsPoint: aPoint) or: [
		self stateIcon notNil and: [self stateIconBounds containsPoint: aPoint]]! !

!MergeJoinSection methodsFor: 'as yet unclassified' stamp: 'FernandoOlivero 4/12/2011 09:54'!
defaultStateIcons
	"Answer the default state icons."
	| uiTheme |
	uiTheme :=self theme.
	^{uiTheme  smallBackIcon.
		uiTheme  smallForwardIcon.
		uiTheme  smallOkIcon.
		uiTheme  smallCancelIcon}! !

!MergeJoinSection methodsFor: 'as yet unclassified' stamp: 'MarcusDenker 4/29/2011 00:36'!
drawOn: aCanvas
	"Draw the join on the given canvas."
	
	super drawOn: aCanvas.
	self stateIcon ifNotNil: [:i |
		aCanvas
			translucentImage: i
			at: self stateIconBounds topLeft]! !

!MergeJoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 1/8/2009 18:05'!
initialize
	"Initialize the receiver."

	self
		allowClick: true;
		selected: false;
		selectionState: #dst;
		stateIcons: self defaultStateIcons;
		selectedBorderColor: Color black.
	super initialize! !

!MergeJoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 10/31/2006 13:03'!
selectNextState
	"Set the selection state to the next one with wraparound."

	self selectionState: (
		self selectionStates at: (
			(self selectionStates indexOf: self selectionState) \\
				self selectionStates size + 1))! !

!MergeJoinSection methodsFor: 'accessing' stamp: 'gvc 10/25/2006 17:47'!
selected
	"Answer the value of selected"

	^ selected! !

!MergeJoinSection methodsFor: 'accessing' stamp: 'gvc 10/30/2006 15:25'!
selected: aBoolean
	"Set the value of selected"

	selected := aBoolean.
	self
		updateHighlights;
		changed: #selected! !

!MergeJoinSection methodsFor: 'accessing' stamp: 'gvc 10/25/2006 17:43'!
selectedBorderColor
	"Answer the value of selectedBorderColor"

	^ selectedBorderColor! !

!MergeJoinSection methodsFor: 'accessing' stamp: 'gvc 10/25/2006 17:43'!
selectedBorderColor: anObject
	"Set the value of selectedBorderColor"

	selectedBorderColor := anObject! !

!MergeJoinSection methodsFor: 'accessing' stamp: 'gvc 10/31/2006 13:00'!
selectionState
	"Answer the value of selectionState"

	^ selectionState! !

!MergeJoinSection methodsFor: 'accessing' stamp: 'gvc 10/31/2006 13:00'!
selectionState: anObject
	"Set the value of selectionState"

	selectionState := anObject! !

!MergeJoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 10/31/2006 13:01'!
selectionStates
	"Answer the valid selection states in order."

	^#(src dst both neither)! !

!MergeJoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 7/6/2007 15:14'!
stateIcon
	"Answer the state icon to use."

	self wantsClick ifFalse: [^nil].
	^self stateIcons at: (self selectionStates indexOf: self selectionState)! !

!MergeJoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 7/30/2009 13:44'!
stateIconBounds
	"Answer the bounds of the state icon."
	
	|i|
	i := self stateIcon ifNil: [^nil].
	^self shape bounds center - (i extent // 2)
		extent: i extent! !

!MergeJoinSection methodsFor: 'accessing' stamp: 'gvc 10/31/2006 13:03'!
stateIcons
	"Answer the value of stateIcons"

	^ stateIcons! !

!MergeJoinSection methodsFor: 'accessing' stamp: 'gvc 10/31/2006 13:03'!
stateIcons: anObject
	"Set the value of stateIcons"

	stateIcons := anObject! !

!MergeJoinSection methodsFor: 'as yet unclassified' stamp: 'MarcusDenker 4/14/2011 15:38'!
theme
	"Answer the ui theme that provides controls."

	^UITheme current! !

!MergeJoinSection methodsFor: 'as yet unclassified' stamp: 'gvc 1/8/2009 18:04'!
wantsClick
	"Allow if explictly enabled and super."

	^self allowClick and: [super wantsClick]! !

Object subclass: #JoinSide
	instanceVariableNames: 'range offset lineRange highlights color text'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-Tools-Diff'!

!JoinSide methodsFor: 'accessing' stamp: 'gvc 11/1/2006 10:41'!
color
	"Answer the value of color"

	^ color! !

!JoinSide methodsFor: 'accessing' stamp: 'gvc 11/1/2006 10:41'!
color: anObject
	"Set the value of color"

	color := anObject! !

!JoinSide methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 10:58'!
highlight
	"Answer the primary highlight."

	^(self highlights ifEmpty: [^nil]) first! !

!JoinSide methodsFor: 'accessing' stamp: 'gvc 11/1/2006 10:41'!
highlights
	"Answer the value of highlights"

	^ highlights! !

!JoinSide methodsFor: 'accessing' stamp: 'gvc 11/1/2006 10:41'!
highlights: anObject
	"Set the value of highlights"

	highlights := anObject! !

!JoinSide methodsFor: 'as yet unclassified' stamp: 'gvc 11/1/2006 11:09'!
initialize
	"Initialize the receiver."

	super initialize.
	self
		highlights: #();
		offset: 0@0;
		range: (1 to: 1);
		lineRange: (1 to: 0);
		color: Color yellow;
		text: ''! !

!JoinSide methodsFor: 'accessing' stamp: 'gvc 11/1/2006 10:41'!
lineRange
	"Answer the value of lineRange"

	^ lineRange! !

!JoinSide methodsFor: 'accessing' stamp: 'gvc 11/1/2006 10:41'!
lineRange: anObject
	"Set the value of lineRange"

	lineRange := anObject! !

!JoinSide methodsFor: 'accessing' stamp: 'gvc 11/1/2006 10:41'!
offset
	"Answer the value of offset"

	^ offset! !

!JoinSide methodsFor: 'accessing' stamp: 'gvc 11/1/2006 10:41'!
offset: anObject
	"Set the value of offset"

	offset := anObject! !

!JoinSide methodsFor: 'accessing' stamp: 'gvc 11/1/2006 10:41'!
range
	"Answer the value of range"

	^ range! !

!JoinSide methodsFor: 'accessing' stamp: 'gvc 11/1/2006 10:41'!
range: anObject
	"Set the value of range"

	range := anObject! !

!JoinSide methodsFor: 'accessing' stamp: 'gvc 11/1/2006 11:09'!
text
	"Answer the value of text"

	^ text! !

!JoinSide methodsFor: 'accessing' stamp: 'gvc 11/1/2006 11:09'!
text: anObject
	"Set the value of text"

	text := anObject! !

!MCWorkingCopyBrowser methodsFor: '*Polymorph-Tools-Diff' stamp: 'SeanDeNigris 6/21/2012 09:07'!
viewChanges
	"View the changes made in the working copy."
	
	| patch |
	'Finding changes' displayProgressFrom: 0 to: 10 during:[:bar|
		self canSave ifTrue:[
		bar current: 1.
		patch := workingCopy changesRelativeToRepository: self repository].
		patch isNil ifTrue: [^ self].
		bar current: 3.
		patch isEmpty
			ifTrue: [
				workingCopy modified: false.
				bar current: 10.
				self inform: 'No changes' ]
			ifFalse: [
				workingCopy modified: true.
				bar current: 5.
				self
					viewChanges: patch
					from: workingCopy  packageName, ' (', workingCopy ancestry ancestorString, ')'
					to: ('Modified {1}' translated format: {workingCopy description})]]! !

!MCWorkingCopyBrowser methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 2/9/2010 14:06'!
viewChanges: patch from: fromDescription to: toDescription
	"Open a browser on the given patch."

	PSMCPatchMorph usedByDefault
		ifTrue: [((PSMCPatchMorph forPatch: patch)
						fromDescription: fromDescription;
						toDescription: toDescription;
						newWindow)
					title: ('Changes to {1}' translated format: {fromDescription});
					open]
		ifFalse: [(MCPatchBrowser forPatch: patch)
					label: 'Patch Browser: ', workingCopy description;
					show]! !

!MCClassDefinition methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 1/14/2009 13:41'!
classDefinitionString
	"Answer a string describing the class-side definition."
	
	^String streamContents: [:stream | self printClassDefinitionOn: stream]! !

!MCClassDefinition methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 1/14/2009 13:41'!
printClassDefinitionOn: stream
	"Print a class-side definition of the receiver on the given stream.
	Class instance variables and class traits."
	
		stream
			nextPutAll: self className;
			nextPutAll: ' class';
			cr; tab.
		self hasClassTraitComposition ifTrue: [
			stream 
				nextPutAll: 'uses: ';
		 		nextPutAll: self classTraitCompositionString;
				cr; tab ].
		stream
			nextPutAll: 'instanceVariableNames: ';
			store: self classInstanceVariablesString! !

!MCOrganizationDefinition methodsFor: '*Polymorph-Tools-Diff' stamp: 'gvc 10/30/2006 11:18'!
patchWrapper
	"Answer a wrapper for a patch tree for the receiver."

	^PSMCOrganizationChangeWrapper with: self! !

!MCOrganizationDefinition methodsFor: '*Polymorph-Tools-Diff' stamp: 'BenjaminVanRyseghem 9/28/2011 15:24'!
shortSummaryPrefix

	^ self definition description last! !
